<?php

include_once(_PS_MODULE_DIR_ . 'axepta/libraries/Helper.php');

if (!defined('_PS_VERSION_')) {
    exit;
}

/**
 * Class Axepta
 *
 * @property string name
 * @property string tab
 * @property string version
 * @property string author
 * @property int need_instance
 * @property bool currencies
 * @property string currencies_mode
 * @property array ps_versions_compliancy
 * @property int is_eu_compatible
 * @property string displayName
 * @property string description
 * @property string confirm_uninstall
 */
class Axepta extends PaymentModule
{
    /**
     * Sets the Information for the Module manager
     * Also creates an instance of this class
     */
    public function __construct()
    {
        $this->name = 'axepta';
        $this->displayName = 'Axepta BNP Paribas';
        $this->tab = 'payments_gateways';
        $this->version = '1.1.0';
        $this->author = 'Axepta BNP Paribas';
        $this->need_instance = 1;
        $this->currencies = true;
        $this->currencies_mode = 'checkbox';
        $this->ps_versions_compliancy = [
            'min' => '1.7'
        ];
        $this->is_eu_compatible = 1;

        parent::__construct();

        $this->displayName = $this->l('Axepta BNP Paribas');
        $this->description = $this->l('Accept payments via Axepta BNP Paribas provider');
        $this->confirm_uninstall = $this->l('Are you sure you want to uninstall Axepta BNP Paribas module?');
    }

    /**
     * @param string $name
     * @param string $type
     *
     * @return string
     */
    public static function buildTemplatePath($name, $type)
    {
        return 'module:axepta/views/templates/' . $type . '/' . $name . '.tpl';
    }

    /**
     * This function installs the Axepta Module
     *
     * @return boolean
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function install()
    {

        if (!(
            parent::install()
            && $this->registerHook('paymentOptions')
            && $this->registerHook('orderConfirmation')
            && $this->registerHook('paymentReturn')
            && $this->registerHook('displayBackOfficeHeader')
            && $this->registerHook('displayHeader')
            && $this->registerHook('displayReassurance')

            && Configuration::updateValue('AXEPTA_SANDBOX', 0)

            && Configuration::updateValue('AXETA_AUTHORIZATION_TOKEN', '')
            && Configuration::updateValue('AXEPTA_SERVICE_ID', '')
            && Configuration::updateValue('AXEPTA_SERVICE_KEY', '')
            && Configuration::updateValue('AXEPTA_MERCHANT_ID', '')

            && Configuration::updateValue('AXEPTA_CREDIT_TYPE', '')

            && Configuration::updateValue('AXEPTA_GA_KEY', '')
            && Configuration::updateValue('AXEPTA_PAYMENT_TITLE', '')
        )) {

            return false;
        }

        if (Validate::isInt(Configuration::get('PAYMENT_AXEPTA_NEW_STATUS'))
            xor (Validate::isLoadedObject($orderStateNew = new OrderState(Configuration::get('PAYMENT_AXEPTA_NEW_STATUS'))))) {

            $orderStateNew = new OrderState();
            $missingLang = true;

            $langs = [
                'en' => 'Payment Axepta BNP Paribas: awaiting for confirmation',
                'pl' => 'Płatność Axepta BNP Paribas: oczekuje na potwierdzenie',
            ];

            foreach ($langs as $lang => $message) {
                $langId = Language::getIdByIso($lang);
                if (isset($langId)) {
                    $orderStateNew->name[$langId] = $message;
                    $missingLang = false;
                }
            }

            if ($missingLang) {
                $langId = (int)$this->context->language->id;
                $orderStateNew->name[$langId] = $langs['en'];
            }

            $orderStateNew->send_email = false;
            $orderStateNew->invoice = false;
            $orderStateNew->unremovable = false;
            $orderStateNew->color = "lightblue";

            if (!$orderStateNew->add()) {
                $this->_errors[] = $this->l('There was an Error installing the module. Cannot add new order state.');

                return false;
            }
            if (!Configuration::updateValue('PAYMENT_AXEPTA_NEW_STATUS', $orderStateNew->id)) {
                $this->_errors[] = $this->l('There was an Error installing the module. Cannot update new order state.');

                return false;
            }

            copy(_PS_MODULE_DIR_ . $this->name . '/axepta.gif', _PS_IMG_DIR_ . 'os/' . $orderStateNew->id . '.gif');
        }

        return true;
    }

    /**
     * This function uninstalls the Axepta Module
     *
     * @return boolean
     */
    public function uninstall()
    {
        return Configuration::deleteByName('AXEPTA_SANDBOX')

            && Configuration::deleteByName('AXEPTA_AUTHORIZATION_TOKEN')
            && Configuration::deleteByName('AXEPTA_SERVICE_ID')
            && Configuration::deleteByName('AXEPTA_SERVICE_KEY')
            && Configuration::deleteByName('AXEPTA_MERCHANT_ID')

            && Configuration::deleteByName('AXEPTA_CREDIT_TYPE')

            && Configuration::deleteByName('AXEPTA_GA_KEY')
            && Configuration::deleteByName('AXEPTA_PAYMENT_TITLE')

            && parent::uninstall();
    }

    /**
     * Get Media path.
     *
     * @param string $mediaUri
     * @param null $cssMediaType
     *
     * @return array|bool|string
     */
    public static function getMediaPath($mediaUri, $cssMediaType = null)
    {
        if (is_array($mediaUri) || empty($mediaUri)) {
            return false;
        }

        $urlData = parse_url($mediaUri);
        if (!is_array($urlData)) {
            return false;
        }

        if (!array_key_exists('host', $urlData)) {
            $mediaUriHostMode = '/' . ltrim(str_replace(str_replace([
                    '/',
                    '\\',
                ], DIRECTORY_SEPARATOR, _PS_CORE_DIR_), __PS_BASE_URI__, $mediaUri), '/\\');
            $mediaUri = '/' . ltrim(str_replace(str_replace([
                    '/',
                    '\\',
                ], DIRECTORY_SEPARATOR, _PS_ROOT_DIR_), __PS_BASE_URI__, $mediaUri), '/\\');
            // remove PS_BASE_URI on _PS_ROOT_DIR_ for the following
            $fileUri = _PS_ROOT_DIR_ . Tools::str_replace_once(__PS_BASE_URI__, DIRECTORY_SEPARATOR, $mediaUri);
            $fileUriHostMode = _PS_CORE_DIR_ . Tools::str_replace_once(__PS_BASE_URI__, DIRECTORY_SEPARATOR, Tools::str_replace_once(_PS_CORE_DIR_, '', $mediaUri));

            if (!@filemtime($fileUri) || @filesize($fileUri) === 0) {
                if (!defined('_PS_HOST_MODE_')) {
                    return false;
                } elseif (!@filemtime($fileUriHostMode) || @filesize($fileUriHostMode) === 0) {
                    return false;
                } else {
                    $mediaUri = $mediaUriHostMode;
                }
            }

            $mediaUri = str_replace('//', '/', $mediaUri);
        }

        if ($cssMediaType) {
            return [$mediaUri => $cssMediaType];
        }

        return $mediaUri;
    }

    /**
     * Display configuration form.
     *
     * @return mixed
     */
    public function getContent()
    {

        $msg = '';

        if (Tools::isSubmit('submitAxepta')) {
            if ($this->saveConfiguration()) {
                $msg = [
                    'type' => 'success',
                    'message' => $this->l('Configuration updated successfully'),
                ];
            } else {
                $msg = [
                    'type' => 'error',
                    'message' => $this->l('There was an error saving your configuration'),
                ];
            }
        }

        $this->context->smarty->assign([
            'axepta_form' => './index.php?tab=AdminModules&configure=axepta&token=' . Tools::getAdminTokenLite('AdminModules') . '&tab_module=' . $this->tab . '&module_name=axepta',

            'axepta_enabled' => Configuration::get('AXEPTA_ENABLED'),
            'axepta_sandbox' => Configuration::get('AXEPTA_SANDBOX'),

            'axepta_authorization_token' => Configuration::get('AXEPTA_AUTHORIZATION_TOKEN'),
            'axepta_service_id' => Configuration::get('AXEPTA_SERVICE_ID'),
            'axepta_service_key' => Configuration::get('AXEPTA_SERVICE_KEY'),
            'axepta_merchant_id' => Configuration::get('AXEPTA_MERCHANT_ID'),

            'axepta_credit_type' => Configuration::get('AXEPTA_CREDIT_TYPE'),

            'axepta_notification_url' => $this->context->link->getModuleLink('axepta', 'notification'),
            'axepta_msg' => $msg,
            'axepta_ga_key' => Configuration::get('AXEPTA_GA_KEY'),

            'axepta_payment_title' => Configuration::get('AXEPTA_PAYMENT_TITLE'),
            'axepta_payment_title_default' => $this->getPaymentTitleDefault(),
        ]);

        return $this->fetchTemplate('admin.tpl');
    }

    /**
     * @return bool
     */
    private function saveConfiguration()
    {

        Configuration::updateValue('AXEPTA_SANDBOX', pSQL(Tools::getValue('axepta_sandbox')));

        Configuration::updateValue('AXEPTA_AUTHORIZATION_TOKEN', pSQL(Tools::getValue('axepta_authorization_token')));
        Configuration::updateValue('AXEPTA_SERVICE_ID', pSQL(Tools::getValue('axepta_service_id')));
        Configuration::updateValue('AXEPTA_SERVICE_KEY', pSQL(Tools::getValue('axepta_service_key')));
        Configuration::updateValue('AXEPTA_MERCHANT_ID', pSQL(Tools::getValue('axepta_merchant_id')));

        Configuration::updateValue('AXEPTA_CREDIT_TYPE', pSQL(Tools::getValue('axepta_credit_type')));

        Configuration::updateValue('AXEPTA_GA_KEY', pSQL(Tools::getValue('axepta_ga_key')));
        Configuration::updateValue('AXEPTA_PAYMENT_TITLE', pSQL(Tools::getValue('axepta_payment_title')));

        return true;
    }

    /**
     * @return string
     */
    private function getPaymentTitleDefault()
    {
        return $this->l('Axepta BNP Paribas');
    }

    /**
     * @param string $name
     *
     * @return mixed
     */
    public function fetchTemplate($name)
    {
        return $this->display(__FILE__, $name);
    }

    /**
     * Hook that run on mobile version of Prestashop.
     */
    public function hookDisplayMobileHeader()
    {
        $this->hookHeader();
    }

    /**
     * Hook that adds css for old versions of Prestashop.
     */
    public function hookHeader()
    {
        $this->context->controller->addCSS(($this->_path) . 'assets/css/axepta-legacy-front.min.css', 'all');
    }

    /**
     * Hook that adds css and js in admin panel. For newer version of PrestaShop.
     */
    public function hookBackOfficeHeader()
    {
        $this->context->controller->addJquery();
        $this->context->controller->addCSS($this->_path . 'assets/css/axepta-admin.min.css', 'all');
        $this->context->controller->addCSS($this->_path . 'assets/css/font-awesome.min.css', 'all');

        if (!Configuration::get('AXEPTA_AUTHORIZATION_TOKEN')
            || !Configuration::get('AXEPTA_MERCHANT_ID')
            || !Configuration::get('AXEPTA_SERVICE_ID')) {
            return '';
        }

        $orderId = Tools::getValue('id_order');

        if ($orderId === false) {
            return '';
        }

        $order = new Order($orderId);
        $orderCurrentState = $order->getCurrentState();

        if ($order->module !== $this->name
            || ($orderCurrentState === Configuration::get('PAYMENT_AXEPTA_NEW_STATUS'))
        ) {
            return '';
        }

        $orderPayment = $order->getOrderPaymentCollection()->getResults()[0];

        if (!isset($orderPayment->transaction_id)
            || !$orderPayment->transaction_id
        ) {
            return '';
        }

        $api = new \Axepta\Payment\Api(
            Configuration::get('AXEPTA_AUTHORIZATION_TOKEN'),
            Configuration::get('AXEPTA_MERCHANT_ID'),
            Configuration::get('AXEPTA_SERVICE_ID'),
            self::getEnvironment()
        );

        $apiTransaction = $this->getApiTransaction($orderPayment->transaction_id);

        if (!$apiTransaction
            || $apiTransaction['amountPaid'] === $apiTransaction['amountRefunded']) {
            return '';
        }

        $amountToRefund = $apiTransaction['amountPaid'] - $apiTransaction['amountSubmittedRefund'];

        $msg = '';
        $refundError = '';

        $axeptaSubmitRefund = Tools::getValue('axepta_submit_refund');
        $axeptaRefundAmount = Tools::getValue('axepta_refund_amount');

        $this->context->smarty->assign('axepta_amount_refundable', \Axepta\Payment\Util::convertAmountToMain($amountToRefund));
        $this->context->smarty->assign('new_layout_refund', version_compare(_PS_VERSION_, '1.7.7.0', '>='));
        $this->context->smarty->assign(
            'axepta_submit_to_refund_amount',
            \Axepta\Payment\Util::convertAmountToMain(abs($apiTransaction['amountRefunded'] - $apiTransaction['amountSubmittedRefund']))
            . ' '
            . $apiTransaction['currency']
        );
        $this->context->smarty->assign(
            'axepta_refunded_amount',
            \Axepta\Payment\Util::convertAmountToMain($apiTransaction['amountRefunded'])
            . ' '
            . $apiTransaction['currency']
        );

        if (!$axeptaSubmitRefund || !$axeptaRefundAmount) {

            $this->context->smarty->assign('axepta_refund_message', $msg);
            $this->context->smarty->assign('axepta_refund_error', $refundError);

            return $this->fetchTemplate('/views/templates/admin/refunds.tpl');
        }

        $refund = $api->createRefund(
            $api->prepareRefundData(
                \Axepta\Payment\Util::convertAmountToFractional($axeptaRefundAmount)
            ),
            $orderPayment->transaction_id
        );

        if (!$refund['success']) {

            $refundError = $this->l('Refund error.');

            PrestaShopLogger::addLog($this->l('Refund error.') . ' ' . $this->l('Order ID:') . ' ' . $orderId);

            $this->context->smarty->assign('axepta_refund_message', $msg);
            $this->context->smarty->assign('axepta_refund_error', $refundError);

            return $this->fetchTemplate('/views/templates/admin/refunds.tpl');
        }

        $currencyInfo = Currency::getCurrency($order->id_currency);

        $msg = $this->l('Refund for amount:')
            . ' '
            . round($axeptaRefundAmount, 2)
            . ' '
            . $currencyInfo['iso_code']
            . ' '
            . $this->l('has been requested and waiting for completion.')
            . '.';

        PrestaShopLogger::addLog($msg);

        $apiTransaction = $this->getApiTransaction($orderPayment->transaction_id);

        if (!$apiTransaction
            || $apiTransaction['amountPaid'] === $apiTransaction['amountRefunded']) {
            return '';
        }

        $amountToRefund = $apiTransaction['amountPaid'] - $apiTransaction['amountSubmittedRefund'];

        $this->context->smarty->assign('axepta_amount_refundable', \Axepta\Payment\Util::convertAmountToMain($amountToRefund));
        $this->context->smarty->assign('axepta_refund_message', $msg);
        $this->context->smarty->assign('axepta_refund_error', $refundError);
        $this->context->smarty->assign(
            'axepta_submit_to_refund_amount',
            \Axepta\Payment\Util::convertAmountToMain(abs($apiTransaction['amountRefunded'] - $apiTransaction['amountSubmittedRefund']))
            . ' '
            . $apiTransaction['currency']
        );
        $this->context->smarty->assign(
            'axepta_refunded_amount',
            \Axepta\Payment\Util::convertAmountToMain($apiTransaction['amountRefunded'])
            . ' '
            . $apiTransaction['currency']
        );

        return $this->fetchTemplate('/views/templates/admin/refunds.tpl');
    }

    /**
     * @return string
     */
    public static function getEnvironment()
    {
        return Configuration::get('AXEPTA_SANDBOX')
            ? \Axepta\Payment\Util::ENVIRONMENT_SANDBOX
            : \Axepta\Payment\Util::ENVIRONMENT_PRODUCTION;
    }

    /**
     * @param string $id
     *
     * @return array
     */
    private function getApiTransaction($id)
    {

        $api = new \Axepta\Payment\Api(
            Configuration::get('AXEPTA_AUTHORIZATION_TOKEN'),
            Configuration::get('AXEPTA_MERCHANT_ID'),
            Configuration::get('AXEPTA_SERVICE_ID'),
            self::getEnvironment()
        );

        $apiTransaction = $api->getTransaction($id);

        if (!$apiTransaction['success']) {
            return [];
        }

        return $apiTransaction['body']['data']['payment'];
    }

    /**
     * Hook that adds css and js in admin panel.
     */
    public function hookBackOfficeFooter()
    {
        echo '<link type="text/css" rel="stylesheet" href="' . $this->_path . 'assets/css/axepta-admin.min.css">';
        echo '<link type="text/css" rel="stylesheet" href="' . $this->_path . 'assets/font-awesome.min.css">';

        echo '<script src="' . $this->_path . 'assets/js/axepta-legacy-admin.js"></script>';
    }

    /**
     * @param array $params
     *
     * @return array|bool
     * @throws SmartyException
     * @throws Exception
     */
    public function hookPaymentOptions($params)
    {
        if (
            !Configuration::get('AXEPTA_MERCHANT_ID')
            || !Configuration::get('AXEPTA_SERVICE_ID')
            || !Configuration::get('AXEPTA_SERVICE_KEY')
            || !$this->active
        ) {
            return false;
        }

        $environment = Configuration::get('AXEPTA_SANDBOX')
            ? \Axepta\Payment\Util::ENVIRONMENT_SANDBOX
            : \Axepta\Payment\Util::ENVIRONMENT_PRODUCTION;

        $link = $this->context->link->getModuleLink('axepta', 'payment');

        $this->context->smarty->assign([
            'axepta_ps_version' => _PS_VERSION_,
            'axepta_enabled' => Configuration::get('AXEPTA_ENABLED'),
            'axepta_sandbox' => Configuration::get('AXEPTA_SANDBOX'),
            'loading_gif' => self::getMediaPath(_PS_MODULE_DIR_ . $this->name . '/assets/img/loading.gif'),
            'axepta_payment_url' => $link,
        ]);

        $paymentOptions = [];

        $paymentOptionAxepta = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $paymentOptions[] = $paymentOptionAxepta->setCallToActionText(
            (Configuration::get('AXEPTA_PAYMENT_TITLE')
                ?: $this->getPaymentTitleDefault()) . ($environment === \Axepta\Payment\Util::ENVIRONMENT_SANDBOX
                ? ' ' . $this->l('Sandbox is enabled.')
                : ''))
            ->setModuleName($this->name)
            ->setAction($link)
            ->setLogo($this->getLogoPath(
                'logo-mini.png'
            ));

        return $paymentOptions;
    }

    /**
     * @param string $file
     *
     * @return array|bool|string
     */
    public function getLogoPath($file)
    {

        return self::getMediaPath(_PS_MODULE_DIR_ . $this->name . '/assets/img/' . $file);
    }

    /**
     * @return mixed
     */
    public function hookPaymentReturn()
    {

        return null;
    }

    /**
     * Prepares data for form that is sent to Axepta.
     *
     * @param Cart $cart
     *
     * @return array
     * @throws Exception
     */
    public function getDataForRequestToPaywall($cart)
    {

        $cartId = $cart->id;

        $customer = new Customer(intval($cart->id_customer));
        $addressBilling = new Address((int)$cart->id_address_invoice);
        $addressDelivery = new Address((int)$cart->id_address_delivery);

        $currencyInfo = Currency::getCurrency($cart->id_currency);

        $total = $cart->getOrderTotal();

        $this->validateOrder($cartId,
            Configuration::get('PAYMENT_AXEPTA_NEW_STATUS'),
            $total,
            $this->displayName,
            null,
            [],
            null,
            false,
            $customer->secure_key
        );

        $orderId = Order::getIdByCartId(($cart->id));

        return \Axepta\Payment\Paywall::prepareData(
            \Axepta\Payment\Util::convertAmountToFractional($total),
            $currencyInfo['iso_code'],
            $orderId,
            $customer->firstname,
            $customer->lastname,
            $customer->email,
            ($addressDelivery->phone_mobile
                ?: ($addressBilling->phone_mobile
                    ?: '')),
            _PS_BASE_URL_ . __PS_BASE_URI__ . 'index.php?controller=order-confirmation&id_cart=' . $cart->id . '&id_module=' . $this->id . '&id_order=' . $orderId . '&key=' . $customer->secure_key,
            $this->context->link->getModuleLink('axepta', 'failure'),
            $this->context->link->getModuleLink('axepta', 'confirmation'),
            Configuration::get('AXEPTA_CREDIT_TYPE'),
            $this->getCart($cart),
            $this->version . ';prestashop_' . _PS_VERSION_
        );
    }

    /**
     * @param Cart $cart
     *
     * @return string
     */
    public function getCart($cart)
    {

        $cartData = new \Axepta\Payment\CartData();

        foreach ($cart->getProducts() as $product) {

            $cartData->addItem(
                $product['name'],
                \Axepta\Payment\Util::convertAmountToFractional(round($product['total_wt'], 2) / $product['quantity']),
                $product['quantity']
            );
        }

        $addressBilling = new Address((int)$cart->id_address_invoice);

        $cartData->setAddressBilling(
            $addressBilling->city,
            $addressBilling->address1 . ' ' . $addressBilling->address2,
            CountryCore::getIsoById($addressBilling->id_country),
            $addressBilling->postcode
        );

        $addressDelivery = new Address((int)$cart->id_address_delivery);

        $cartData->setAddressDelivery(
            $addressDelivery->city,
            $addressDelivery->address1 . ' ' . $addressDelivery->address2,
            CountryCore::getIsoById($addressDelivery->id_country),
            $addressDelivery->postcode
        );

        $cartData->setGrandTotal(\Axepta\Payment\Util::convertAmountToFractional($cart->getOrderTotal()));

        return $cartData->prepareCartData();
    }
}
